@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +18,8 @@
  * Boston, MA  02110-1301, USA.
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -28,12 +29,12 @@
 #include "tracker-ontology.h"
 #include "tracker-property.h"
 
-#define XSD_BOOLEAN TRACKER_XSD_PREFIX "boolean"
-#define XSD_DATE TRACKER_XSD_PREFIX "date"
+#define XSD_BOOLEAN  TRACKER_XSD_PREFIX "boolean"
+#define XSD_DATE     TRACKER_XSD_PREFIX "date"
 #define XSD_DATETIME TRACKER_XSD_PREFIX "dateTime"
-#define XSD_DOUBLE TRACKER_XSD_PREFIX "double"
-#define XSD_INTEGER TRACKER_XSD_PREFIX "integer"
-#define XSD_STRING TRACKER_XSD_PREFIX "string"
+#define XSD_DOUBLE   TRACKER_XSD_PREFIX "double"
+#define XSD_INTEGER  TRACKER_XSD_PREFIX "integer"
+#define XSD_STRING   TRACKER_XSD_PREFIX "string"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_PROPERTY, TrackerPropertyPriv))
 
@@ -53,19 +54,20 @@ struct _TrackerPropertyPriv {
 	gboolean       multiple_values;
 	gboolean       filtered;
 	gboolean       transient;
+	gboolean       is_inverse_functional_property;
 
 	GArray        *super_properties;
 };
 
 static void property_finalize     (GObject      *object);
 static void property_get_property (GObject      *object,
-				guint	      param_id,
-				GValue	     *value,
-				GParamSpec   *pspec);
+				   guint         param_id,
+				   GValue       *value,
+				   GParamSpec   *pspec);
 static void property_set_property (GObject      *object,
-				guint	      param_id,
-				const GValue *value,
-				GParamSpec   *pspec);
+				   guint         param_id,
+				   const GValue *value,
+				   GParamSpec   *pspec);
 
 enum {
 	PROP_0,
@@ -80,7 +82,8 @@ enum {
 	PROP_EMBEDDED,
 	PROP_MULTIPLE_VALUES,
 	PROP_FILTERED,
-	PROP_TRANSIENT
+	PROP_TRANSIENT,
+	PROP_IS_INVERSE_FUNCTIONAL_PROPERTY
 };
 
 GType
@@ -90,6 +93,9 @@ tracker_property_type_get_type (void)
 
 	if (etype == 0) {
 		static const GEnumValue values[] = {
+			{ TRACKER_PROPERTY_TYPE_UNKNOWN,
+			  "TRACKER_PROPERTY_TYPE_UNKNOWN",
+			  "unknown" },
 			{ TRACKER_PROPERTY_TYPE_STRING,
 			  "TRACKER_PROPERTY_TYPE_STRING",
 			  "string" },
@@ -108,18 +114,9 @@ tracker_property_type_get_type (void)
 			{ TRACKER_PROPERTY_TYPE_DATETIME,
 			  "TRACKER_PROPERTY_TYPE_DATETIME",
 			  "datetime" },
-			{ TRACKER_PROPERTY_TYPE_BLOB,
-			  "TRACKER_PROPERTY_TYPE_BLOB",
-			  "blob" },
-			{ TRACKER_PROPERTY_TYPE_STRUCT,
-			  "TRACKER_PROPERTY_TYPE_STRUCT",
-			  "struct" },
 			{ TRACKER_PROPERTY_TYPE_RESOURCE,
 			  "TRACKER_PROPERTY_TYPE_RESOURCE",
 			  "resource" },
-			{ TRACKER_PROPERTY_TYPE_FULLTEXT,
-			  "TRACKER_PROPERTY_TYPE_FULLTEXT",
-			  "fulltext" },
 			{ 0, NULL, NULL }
 		};
 
@@ -134,7 +131,7 @@ G_DEFINE_TYPE (TrackerProperty, tracker_property, G_TYPE_OBJECT);
 const gchar *
 tracker_property_type_to_string (TrackerPropertyType fieldtype)
 {
-	GType	    type;
+	GType type;
 	GEnumClass *enum_class;
 	GEnumValue *enum_value;
 
@@ -245,6 +242,14 @@ tracker_property_class_init (TrackerPropertyClass *klass)
 							       "Transient",
 							       FALSE,
 							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_IS_INVERSE_FUNCTIONAL_PROPERTY,
+					 g_param_spec_boolean ("is-inverse-functional-property",
+							       "is-inverse-functional-property",
+							       "Is inverse functional property",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 
 	g_type_class_add_private (object_class, sizeof (TrackerPropertyPriv));
 }
@@ -281,9 +286,9 @@ property_finalize (GObject *object)
 
 static void
 property_get_property (GObject    *object,
-		    guint	param_id,
-		    GValue     *value,
-		    GParamSpec *pspec)
+		       guint	   param_id,
+		       GValue     *value,
+		       GParamSpec *pspec)
 {
 	TrackerPropertyPriv *priv;
 
@@ -326,6 +331,9 @@ property_get_property (GObject    *object,
 	case PROP_TRANSIENT:
 		g_value_set_boolean (value, priv->transient);
 		break;
+	case PROP_IS_INVERSE_FUNCTIONAL_PROPERTY:
+		g_value_set_boolean (value, priv->is_inverse_functional_property);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -333,10 +341,10 @@ property_get_property (GObject    *object,
 }
 
 static void
-property_set_property (GObject	 *object,
-		    guint	  param_id,
-		    const GValue *value,
-		    GParamSpec	 *pspec)
+property_set_property (GObject	    *object,
+		       guint	     param_id,
+		       const GValue *value,
+		       GParamSpec   *pspec)
 {
 	switch (param_id) {
 	case PROP_URI:
@@ -383,6 +391,10 @@ property_set_property (GObject	 *object,
 		tracker_property_set_transient (TRACKER_PROPERTY (object),
 					    g_value_get_boolean (value));
 		break;
+	case PROP_IS_INVERSE_FUNCTIONAL_PROPERTY:
+		tracker_property_set_is_inverse_functional_property (TRACKER_PROPERTY (object),
+		                                                     g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -391,13 +403,13 @@ property_set_property (GObject	 *object,
 
 static gboolean
 field_int_validate (TrackerProperty *field,
-		    const gchar   *property,
-		    gint	    value)
+		    const gchar     *property,
+		    gint	     value)
 {
 #ifdef G_DISABLE_CHECKS
 	GParamSpec *spec;
-	GValue	    value = { 0 };
-	gboolean    valid;
+	GValue value = { 0 };
+	gboolean valid;
 
 	spec = g_object_class_find_property (G_OBJECT_CLASS (field), property);
 	g_return_val_if_fail (spec != NULL, FALSE);
@@ -579,6 +591,18 @@ tracker_property_get_filtered (TrackerProperty *field)
 	return priv->filtered;
 }
 
+gboolean
+tracker_property_get_is_inverse_functional_property (TrackerProperty *property)
+{
+	TrackerPropertyPriv *priv;
+
+	g_return_val_if_fail (TRACKER_IS_PROPERTY (property), FALSE);
+
+	priv = GET_PRIV (property);
+
+	return priv->is_inverse_functional_property;
+}
+
 TrackerProperty **
 tracker_property_get_super_properties (TrackerProperty *property)
 {
@@ -593,7 +617,7 @@ tracker_property_get_super_properties (TrackerProperty *property)
 
 void
 tracker_property_set_uri (TrackerProperty *field,
-		       const gchar  *value)
+			  const gchar     *value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -607,8 +631,8 @@ tracker_property_set_uri (TrackerProperty *field,
 	priv->name = NULL;
 
 	if (value) {
-		gchar *namespace_uri, *hash;
 		TrackerNamespace *namespace;
+		gchar *namespace_uri, *hash;
 
 		priv->uri = g_strdup (value);
 
@@ -636,7 +660,7 @@ tracker_property_set_uri (TrackerProperty *field,
 
 void
 tracker_property_set_transient (TrackerProperty *field,
-			        gboolean value)
+			        gboolean         value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -652,7 +676,7 @@ tracker_property_set_transient (TrackerProperty *field,
 
 void
 tracker_property_set_data_type (TrackerProperty     *field,
-			     TrackerPropertyType  value)
+				TrackerPropertyType  value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -665,8 +689,8 @@ tracker_property_set_data_type (TrackerProperty     *field,
 }
 
 void
-tracker_property_set_domain (TrackerProperty   *field,
-			   TrackerClass *value)
+tracker_property_set_domain (TrackerProperty *field,
+			     TrackerClass    *value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -687,11 +711,11 @@ tracker_property_set_domain (TrackerProperty   *field,
 }
 
 void
-tracker_property_set_range (TrackerProperty   *property,
-			   TrackerClass *value)
+tracker_property_set_range (TrackerProperty *property,
+			   TrackerClass     *value)
 {
 	TrackerPropertyPriv *priv;
-	const gchar         *range_uri;
+	const gchar *range_uri;
 
 	g_return_if_fail (TRACKER_IS_PROPERTY (property));
 	g_return_if_fail (TRACKER_IS_CLASS (value));
@@ -726,7 +750,7 @@ tracker_property_set_range (TrackerProperty   *property,
 
 void
 tracker_property_set_weight (TrackerProperty *field,
-			  gint		value)
+			     gint	      value)
 {
 	TrackerPropertyPriv *priv;
 	g_return_if_fail (TRACKER_IS_PROPERTY (field));
@@ -743,7 +767,7 @@ tracker_property_set_weight (TrackerProperty *field,
 
 void
 tracker_property_set_indexed (TrackerProperty *field,
-			   gboolean	  value)
+			      gboolean         value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -757,7 +781,7 @@ tracker_property_set_indexed (TrackerProperty *field,
 
 void
 tracker_property_set_fulltext_indexed (TrackerProperty *field,
-				    gboolean	  value)
+				       gboolean	        value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -771,7 +795,7 @@ tracker_property_set_fulltext_indexed (TrackerProperty *field,
 
 void
 tracker_property_set_embedded (TrackerProperty *field,
-			    gboolean	  value)
+			       gboolean	        value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -785,7 +809,7 @@ tracker_property_set_embedded (TrackerProperty *field,
 
 void
 tracker_property_set_multiple_values (TrackerProperty *field,
-				   gboolean	 value)
+				      gboolean         value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -804,7 +828,7 @@ tracker_property_set_multiple_values (TrackerProperty *field,
 
 void
 tracker_property_set_filtered (TrackerProperty *field,
-			    gboolean	  value)
+			       gboolean	        value)
 {
 	TrackerPropertyPriv *priv;
 
@@ -817,11 +841,25 @@ tracker_property_set_filtered (TrackerProperty *field,
 }
 
 void
-tracker_property_set_super_properties (TrackerProperty *property,
-			         TrackerProperty **value)
+tracker_property_set_is_inverse_functional_property (TrackerProperty *property,
+                                                     gboolean         value)
 {
 	TrackerPropertyPriv *priv;
-	TrackerProperty     **super_property;
+
+	g_return_if_fail (TRACKER_IS_PROPERTY (property));
+
+	priv = GET_PRIV (property);
+
+	priv->is_inverse_functional_property = value;
+	g_object_notify (G_OBJECT (property), "is-inverse-functional-property");
+}
+
+void
+tracker_property_set_super_properties (TrackerProperty  *property,
+				       TrackerProperty **value)
+{
+	TrackerPropertyPriv *priv;
+	TrackerProperty **super_property;
 
 	g_return_if_fail (TRACKER_IS_PROPERTY (property));
 
@@ -837,7 +875,7 @@ tracker_property_set_super_properties (TrackerProperty *property,
 
 void
 tracker_property_add_super_property (TrackerProperty *property,
-			         TrackerProperty *value)
+				     TrackerProperty *value)
 {
 	TrackerPropertyPriv *priv;
 
