@@ -924,6 +924,7 @@ tracker_db_cursor_sqlite_finalize (GObject *object)
 		TrackerDBStatementSqlitePrivate *stmt_priv;
 		stmt_priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE (priv->ref_stmt);
 		stmt_priv->stmt_is_sunk = FALSE;
+		tracker_db_statement_sqlite_reset (priv->ref_stmt);
 		g_object_unref (priv->ref_stmt);
 	} else {
 		sqlite3_finalize (priv->stmt);
@@ -1013,6 +1014,19 @@ tracker_db_statement_sqlite_bind_int64 (TrackerDBStatement	 *stmt,
 	sqlite3_bind_int64 (priv->stmt, index + 1, value);
 }
 
+static void
+tracker_db_statement_sqlite_bind_null (TrackerDBStatement	 *stmt,
+				       int			  index)
+{
+	TrackerDBStatementSqlitePrivate *priv;
+
+	priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE (stmt);
+
+	g_assert (!priv->stmt_is_sunk);
+
+	sqlite3_bind_null (priv->stmt, index + 1);
+}
+
 static void
 tracker_db_statement_sqlite_bind_text (TrackerDBStatement	 *stmt,
 				       int			  index,
@@ -1176,6 +1190,7 @@ tracker_db_statement_sqlite_iface_init (TrackerDBStatementIface *iface)
 	iface->bind_double = tracker_db_statement_sqlite_bind_double;
 	iface->bind_int = tracker_db_statement_sqlite_bind_int;
 	iface->bind_int64 = tracker_db_statement_sqlite_bind_int64;
+	iface->bind_null = tracker_db_statement_sqlite_bind_null;
 	iface->bind_text = tracker_db_statement_sqlite_bind_text;
 	iface->execute = tracker_db_statement_sqlite_execute;
 	iface->start_cursor = tracker_db_statement_sqlite_start_cursor;
