@@ -438,6 +438,15 @@ tracker_db_statement_bind_int64 (TrackerDBStatement	*stmt,
 	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_int64 (stmt, idx, value);
 }
 
+void
+tracker_db_statement_bind_null (TrackerDBStatement	*stmt,
+				int			 idx)
+{
+	g_return_if_fail (TRACKER_IS_DB_STATEMENT (stmt));
+
+	TRACKER_DB_STATEMENT_GET_IFACE (stmt)->bind_null (stmt, idx);
+}
+
 void
 tracker_db_statement_bind_text (TrackerDBStatement	*stmt,
 				int			 idx,
